@@ -348,7 +348,7 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
 {
   GArray *array;
   char **languages;
-  int i;
+  guint i;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   WebKitWebContext *web_context = ephy_embed_shell_get_web_context (shell);
 
@@ -364,14 +364,15 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
       g_array_append_val (array, str);
     }
   }
+  g_strfreev (languages);
 
   ephy_langs_sanitise (array);
 
   webkit_web_context_set_preferred_languages (web_context, (const char * const *)(void *)array->data);
-
-  /* Set preferred languages also for the default web context, used by the sync
-   * tab in the preferences dialog. It doesn't need spellchecking. */
-  webkit_web_context_set_preferred_languages (webkit_web_context_get_default (), (const char * const *)(void *)array->data);
+  /* Used by the Firefox Sync web view in prefs-dialog.c. */
+  g_object_set_data_full (G_OBJECT (web_context), "preferred-languages",
+                          g_strdupv ((char **)(void *)array->data),
+                          (GDestroyNotify)g_strfreev);
 
   if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING)) {
     char **normalized = normalize_languages ((char **)(void *)array->data);
@@ -379,7 +380,8 @@ webkit_pref_callback_accept_languages (GSettings  *settings,
     g_strfreev (normalized);
   }
 
-  g_strfreev (languages);
+  for (i = 0; i < array->len; i++)
+    g_free (g_array_index (array, char *, i));
   g_array_free (array, TRUE);
 }
 
