@@ -30,6 +30,7 @@
 #include "ephy-embed-utils.h"
 #include "ephy-encodings.h"
 #include "ephy-file-helpers.h"
+#include "ephy-filters-manager.h"
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
@@ -51,8 +52,6 @@
 #define PRINT_SETTINGS_FILENAME "print-settings.ini"
 #define OVERVIEW_RELOAD_DELAY 500
 
-#define ADBLOCK_FILTER_UPDATE_FREQUENCY 24 * 60 * 60 /* In seconds */
-
 typedef struct {
   WebKitWebContext *web_context;
   EphyHistoryService *global_history_service;
@@ -62,16 +61,15 @@ typedef struct {
   EphyEmbedShellMode mode;
   WebKitUserContentManager *user_content;
   EphyDownloadsManager *downloads_manager;
-  EphyHostsManager *hosts_manager;
+  EphyPermissionsManager *permissions_manager;
   EphyAboutHandler *about_handler;
   EphyViewSourceHandler *source_handler;
   guint update_overview_timeout_id;
   guint hiding_overview_item;
   GDBusServer *dbus_server;
   GList *web_extensions;
-
-  char *adblock_data_dir;
-  GCancellable *uri_tester_update_cancellable;
+  EphyFiltersManager *filters_manager;
+  GCancellable *cancellable;
 } EphyEmbedShellPrivate;
 
 enum {
@@ -104,9 +102,9 @@ ephy_embed_shell_dispose (GObject *object)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
 
-  if (priv->uri_tester_update_cancellable) {
-    g_cancellable_cancel (priv->uri_tester_update_cancellable);
-    g_clear_object (&priv->uri_tester_update_cancellable);
+  if (priv->cancellable) {
+    g_cancellable_cancel (priv->cancellable);
+    g_clear_object (&priv->cancellable);
   }
 
   if (priv->update_overview_timeout_id > 0) {
@@ -122,23 +120,14 @@ ephy_embed_shell_dispose (GObject *object)
   g_clear_object (&priv->source_handler);
   g_clear_object (&priv->user_content);
   g_clear_object (&priv->downloads_manager);
-  g_clear_object (&priv->hosts_manager);
+  g_clear_object (&priv->permissions_manager);
   g_clear_object (&priv->web_context);
   g_clear_object (&priv->dbus_server);
+  g_clear_object (&priv->filters_manager);
 
   G_OBJECT_CLASS (ephy_embed_shell_parent_class)->dispose (object);
 }
 
-static void
-ephy_embed_shell_finalize (GObject *object)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (EPHY_EMBED_SHELL (object));
-
-  g_free (priv->adblock_data_dir);
-
-  G_OBJECT_CLASS (ephy_embed_shell_parent_class)->finalize (object);
-}
-
 static void
 web_extension_form_auth_data_message_received_cb (WebKitUserContentManager *manager,
                                                   WebKitJavascriptResult   *message,
@@ -146,7 +135,7 @@ web_extension_form_auth_data_message_received_cb (WebKitUserContentManager *mana
 {
   guint request_id;
   guint64 page_id;
-  const char *hostname;
+  const char *origin;
   const char *username;
   GVariant *variant;
   gchar *message_str;
@@ -155,9 +144,9 @@ web_extension_form_auth_data_message_received_cb (WebKitUserContentManager *mana
   variant = g_variant_parse (G_VARIANT_TYPE ("(utss)"), message_str, NULL, NULL, NULL);
   g_free (message_str);
 
-  g_variant_get (variant, "(ut&s&s)", &request_id, &page_id, &hostname, &username);
+  g_variant_get (variant, "(ut&s&s)", &request_id, &page_id, &origin, &username);
   g_signal_emit (shell, signals[FORM_AUTH_DATA_SAVE_REQUESTED], 0,
-                 request_id, page_id, hostname, username);
+                 request_id, page_id, origin, username);
   g_variant_unref (variant);
 }
 
@@ -636,210 +625,6 @@ ftp_request_cb (WebKitURISchemeRequest *request)
   g_object_unref (app_info);
 }
 
-#ifdef HAVE_LIBHTTPSEVERYWHERE
-static void
-https_everywhere_update_cb (HTTPSEverywhereUpdater *updater,
-                            GAsyncResult           *result)
-{
-  GError *error = NULL;
-
-  https_everywhere_updater_update_finish (updater, result, &error);
-
-  if (!error)
-    return;
-
-  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
-      !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_IN_PROGRESS) &&
-      !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_NO_UPDATE_AVAILABLE))
-    g_warning ("Failed to update HTTPS Everywhere rulesets: %s", error->message);
-  g_error_free (error);
-}
-#endif
-
-static const char *
-ephy_embed_shell_ensure_adblock_data_dir (EphyEmbedShell *shell)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-
-  if (priv->adblock_data_dir)
-    return priv->adblock_data_dir;
-
-  /* The filters list is large, so we don't want to store a separate copy per
-   * web app, but users should otherwise be able to configure different filters
-   * per profile directory. */
-  if (priv->mode == EPHY_EMBED_SHELL_MODE_APPLICATION) {
-    char *default_dot_dir = ephy_default_dot_dir ();
-
-    priv->adblock_data_dir = g_build_filename (default_dot_dir, "adblock", NULL);
-    g_free (default_dot_dir);
-  } else
-    priv->adblock_data_dir = g_build_filename (ephy_dot_dir (), "adblock", NULL);
-  g_mkdir_with_parents (priv->adblock_data_dir, 0700);
-
-  return priv->adblock_data_dir;
-}
-
-static gboolean
-adblock_filter_file_is_valid (GFile *file)
-{
-  GFileInfo *file_info;
-  gboolean result = FALSE;
-
-  /* Now check if the local file is too old. */
-  file_info = g_file_query_info (file,
-                                 G_FILE_ATTRIBUTE_TIME_MODIFIED","G_FILE_ATTRIBUTE_STANDARD_SIZE,
-                                 G_FILE_QUERY_INFO_NONE,
-                                 NULL,
-                                 NULL);
-  if (file_info) {
-    if (g_file_info_get_size (file_info) > 0) {
-      GTimeVal current_time;
-      GTimeVal mod_time;
-
-      g_get_current_time (&current_time);
-      g_file_info_get_modification_time (file_info, &mod_time);
-
-      if (current_time.tv_sec > mod_time.tv_sec) {
-        gint64 expire_time = mod_time.tv_sec + ADBLOCK_FILTER_UPDATE_FREQUENCY;
-
-        result = current_time.tv_sec < expire_time;
-      }
-    }
-    g_object_unref (file_info);
-  }
-
-  return result;
-}
-
-typedef struct {
-  EphyEmbedShell *shell;
-
-  GFile *filter_file;
-  GFile *tmp_file;
-} AdblockFilterRetrieveData;
-
-static AdblockFilterRetrieveData *
-adblock_filter_retrieve_data_new (EphyEmbedShell *shell,
-                                  GFile          *filter_file)
-{
-  AdblockFilterRetrieveData* data;
-  char *path, *tmp_path;
-
-  data = g_slice_new (AdblockFilterRetrieveData);
-  data->shell = g_object_ref (shell);
-  data->filter_file = g_object_ref (filter_file);
-
-  path = g_file_get_path (filter_file);
-  tmp_path = g_strdup_printf ("%s.tmp", path);
-  g_free (path);
-  data->tmp_file = g_file_new_for_path (tmp_path);
-  g_free (tmp_path);
-
-  return data;
-}
-
-static void
-adblock_filter_retrieve_data_free (AdblockFilterRetrieveData *data)
-{
-  g_object_unref (data->shell);
-  g_object_unref (data->filter_file);
-  g_object_unref (data->tmp_file);
-
-  g_slice_free (AdblockFilterRetrieveData, data);
-}
-
-static void
-ephy_embed_shell_retrieve_filter_file_finished (GFile                     *src,
-                                                GAsyncResult              *result,
-                                                AdblockFilterRetrieveData *data)
-{
-  GError *error = NULL;
-
-  if (!g_file_copy_finish (src, result, &error) ||
-      !g_file_move (data->tmp_file, data->filter_file, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &error)) {
-    GFileOutputStream *stream;
-
-    /* If failed to retrieve, create an empty file if it doesn't exist to unblock extensions */
-    stream = g_file_create (data->filter_file, G_FILE_CREATE_NONE, NULL, NULL);
-    g_object_unref (stream);
-
-    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-      g_warning ("Error retrieving filter: %s\n", error->message);
-    g_error_free (error);
-  }
-
-  adblock_filter_retrieve_data_free (data);
-}
-
-static void
-ephy_embed_shell_retrieve_filter_file (EphyEmbedShell *shell,
-                                       GFile          *file)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-  GFile *src = g_file_new_for_uri (ADBLOCK_FILTER_URL);
-  AdblockFilterRetrieveData *data;
-
-  if (!priv->uri_tester_update_cancellable)
-    priv->uri_tester_update_cancellable = g_cancellable_new ();
-
-  data = adblock_filter_retrieve_data_new (shell, file);
-
-  g_file_copy_async (src, data->tmp_file,
-                     G_FILE_COPY_OVERWRITE,
-                     G_PRIORITY_DEFAULT,
-                     priv->uri_tester_update_cancellable,
-                     NULL, NULL,
-                     (GAsyncReadyCallback)ephy_embed_shell_retrieve_filter_file_finished,
-                     data);
-
-  g_object_unref (src);
-}
-
-static void
-ephy_embed_shell_update_adblock_filter_file (EphyEmbedShell *shell)
-{
-  GFile *filter_file;
-
-  filter_file = ephy_uri_tester_get_adblock_filter_file (ephy_embed_shell_ensure_adblock_data_dir (shell));
-
-  if (!adblock_filter_file_is_valid (filter_file))
-    ephy_embed_shell_retrieve_filter_file (shell, filter_file);
-
-  g_object_unref (filter_file);
-}
-
-static void
-ephy_embed_shell_update_uri_tester (EphyEmbedShell *shell)
-{
-  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-
-  ephy_embed_shell_update_adblock_filter_file (shell);
-
-  if (priv->mode != EPHY_EMBED_SHELL_MODE_TEST &&
-      priv->mode != EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER) {
-#ifdef HAVE_LIBHTTPSEVERYWHERE
-    HTTPSEverywhereContext *context;
-    HTTPSEverywhereUpdater *updater;
-#endif
-
-    if (!priv->uri_tester_update_cancellable)
-      priv->uri_tester_update_cancellable = g_cancellable_new ();
-
-#ifdef HAVE_LIBHTTPSEVERYWHERE
-    /* We might want to be smarter about this in the future. For now,
-     * trigger an update of the rulesets once each time Epiphany is started. */
-    context = https_everywhere_context_new ();
-    updater = https_everywhere_updater_new (context);
-    https_everywhere_updater_update (updater,
-                                     priv->uri_tester_update_cancellable,
-                                     (GAsyncReadyCallback)https_everywhere_update_cb,
-                                     NULL);
-    g_object_unref (context);
-    g_object_unref (updater);
-#endif
-  }
-}
-
 static void
 web_extension_destroyed (EphyEmbedShell *shell,
                          GObject        *web_extension)
@@ -879,15 +664,30 @@ initialize_web_extensions (WebKitWebContext *web_context,
 
   address = priv->dbus_server ? g_dbus_server_get_client_address (priv->dbus_server) : NULL;
 
-  private_profile = EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode);
+  private_profile = priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE || priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO;
   user_data = g_variant_new ("(msssb)",
                              address,
                              ephy_dot_dir (),
-                             ephy_embed_shell_ensure_adblock_data_dir (shell),
+                             ephy_filters_manager_get_adblock_filters_dir (priv->filters_manager),
                              private_profile);
   webkit_web_context_set_web_extensions_initialization_user_data (web_context, user_data);
 }
 
+static void
+initialize_notification_permissions (WebKitWebContext *web_context,
+                                     EphyEmbedShell   *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+  GList *permitted_origins;
+  GList *denied_origins;
+
+  permitted_origins = ephy_permissions_manager_get_permitted_origins (priv->permissions_manager,
+                                                                      EPHY_PERMISSION_TYPE_SHOW_NOTIFICATIONS);
+  denied_origins = ephy_permissions_manager_get_denied_origins (priv->permissions_manager,
+                                                                EPHY_PERMISSION_TYPE_SHOW_NOTIFICATIONS);
+  webkit_web_context_initialize_notification_permissions (web_context, permitted_origins, denied_origins);
+}
+
 static void
 web_extension_page_created (EphyWebExtensionProxy *extension,
                             guint64                page_id,
@@ -1002,10 +802,15 @@ ephy_embed_shell_create_web_context (EphyEmbedShell *shell)
   char *data_dir;
   char *cache_dir;
 
-  data_dir = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode) ?
+  if (priv->mode == EPHY_EMBED_SHELL_MODE_INCOGNITO) {
+    priv->web_context = webkit_web_context_new_ephemeral ();
+    return;
+  }
+
+  data_dir = g_build_filename (priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE ?
                                ephy_dot_dir () : g_get_user_data_dir (),
                                g_get_prgname (), NULL);
-  cache_dir = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode) ?
+  cache_dir = g_build_filename (priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE ?
                                 ephy_dot_dir () : g_get_user_cache_dir (),
                                 g_get_prgname (), NULL);
 
@@ -1019,6 +824,43 @@ ephy_embed_shell_create_web_context (EphyEmbedShell *shell)
   g_object_unref (manager);
 }
 
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+static void
+https_everywhere_update_cb (HTTPSEverywhereUpdater *updater,
+                            GAsyncResult           *result)
+{
+  GError *error = NULL;
+
+  https_everywhere_updater_update_finish (updater, result, &error);
+
+  if (!error)
+    return;
+
+  if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+      !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_IN_PROGRESS) &&
+      !g_error_matches (error, HTTPS_EVERYWHERE_UPDATE_ERROR, HTTPS_EVERYWHERE_UPDATE_ERROR_NO_UPDATE_AVAILABLE))
+    g_warning ("Failed to update HTTPS Everywhere rulesets: %s", error->message);
+  g_error_free (error);
+}
+#endif
+
+static char *
+adblock_filters_dir (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+  char *result;
+
+  if (priv->mode == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+    char *default_dot_dir = ephy_default_dot_dir ();
+
+    result = g_build_filename (default_dot_dir, "adblock", NULL);
+    g_free (default_dot_dir);
+  } else
+    result = g_build_filename (ephy_dot_dir (), "adblock", NULL);
+
+  return result;
+}
+
 static void
 ephy_embed_shell_startup (GApplication *application)
 {
@@ -1028,12 +870,19 @@ ephy_embed_shell_startup (GApplication *application)
   WebKitCookieManager *cookie_manager;
   char *filename;
   char *cookie_policy;
+  char *filters_dir;
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+  HTTPSEverywhereContext *context;
+  HTTPSEverywhereUpdater *updater;
+#endif
 
   G_APPLICATION_CLASS (ephy_embed_shell_parent_class)->startup (application);
 
-  ephy_embed_shell_update_uri_tester (embed_shell);
+  filters_dir = adblock_filters_dir (shell);
+  priv->filters_manager = ephy_filters_manager_new (filters_dir);
+  g_free (filters_dir);
 
-  ephy_embed_shell_create_web_context (embed_shell);
+  ephy_embed_shell_create_web_context (shell);
 
   ephy_embed_shell_setup_web_extensions_server (shell);
 
@@ -1076,8 +925,13 @@ ephy_embed_shell_startup (GApplication *application)
                     G_CALLBACK (initialize_web_extensions),
                     shell);
 
+  priv->permissions_manager = ephy_permissions_manager_new ();
+  g_signal_connect (priv->web_context, "initialize-notification-permissions",
+                    G_CALLBACK (initialize_notification_permissions),
+                    shell);
+
   /* Favicon Database */
-  favicon_db_path = g_build_filename (EPHY_EMBED_SHELL_MODE_HAS_PRIVATE_PROFILE (priv->mode) ?
+  favicon_db_path = g_build_filename (priv->mode == EPHY_EMBED_SHELL_MODE_PRIVATE ?
                                       ephy_dot_dir () : g_get_user_cache_dir (),
                                       "icondatabase", NULL);
   webkit_web_context_set_favicon_database_directory (priv->web_context, favicon_db_path);
@@ -1116,15 +970,35 @@ ephy_embed_shell_startup (GApplication *application)
 
   /* Store cookies in moz-compatible SQLite format */
   cookie_manager = webkit_web_context_get_cookie_manager (priv->web_context);
-  filename = g_build_filename (ephy_dot_dir (), "cookies.sqlite", NULL);
-  webkit_cookie_manager_set_persistent_storage (cookie_manager, filename,
-                                                WEBKIT_COOKIE_PERSISTENT_STORAGE_SQLITE);
-  g_free (filename);
+  if (priv->mode != EPHY_EMBED_SHELL_MODE_INCOGNITO) {
+    filename = g_build_filename (ephy_dot_dir (), "cookies.sqlite", NULL);
+    webkit_cookie_manager_set_persistent_storage (cookie_manager, filename,
+                                                  WEBKIT_COOKIE_PERSISTENT_STORAGE_SQLITE);
+    g_free (filename);
+  }
 
   cookie_policy = g_settings_get_string (EPHY_SETTINGS_WEB,
                                          EPHY_PREFS_WEB_COOKIES_POLICY);
   ephy_embed_prefs_set_cookie_accept_policy (cookie_manager, cookie_policy);
   g_free (cookie_policy);
+
+#ifdef HAVE_LIBHTTPSEVERYWHERE
+    /* We might want to be smarter about this in the future. For now,
+     * trigger an update of the rulesets once each time Epiphany is started.
+     * Note that the updated rules will not be used until the next time Epiphany
+     * is started. */
+  if (priv->mode != EPHY_EMBED_SHELL_MODE_TEST &&
+      priv->mode != EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER) {
+    context = https_everywhere_context_new ();
+    updater = https_everywhere_updater_new (context);
+    https_everywhere_updater_update (updater,
+                                     priv->cancellable,
+                                     (GAsyncReadyCallback)https_everywhere_update_cb,
+                                     NULL);
+    g_object_unref (context);
+    g_object_unref (updater);
+  }
+#endif
 }
 
 static void
@@ -1223,7 +1097,6 @@ ephy_embed_shell_class_init (EphyEmbedShellClass *klass)
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
 
   object_class->dispose = ephy_embed_shell_dispose;
-  object_class->finalize = ephy_embed_shell_finalize;
   object_class->set_property = ephy_embed_shell_set_property;
   object_class->get_property = ephy_embed_shell_get_property;
   object_class->constructed = ephy_embed_shell_constructed;
@@ -1582,12 +1455,10 @@ ephy_embed_shell_get_downloads_manager (EphyEmbedShell *shell)
   return priv->downloads_manager;
 }
 
-EphyHostsManager *
-ephy_embed_shell_get_hosts_manager (EphyEmbedShell *shell)
+EphyPermissionsManager *
+ephy_embed_shell_get_permissions_manager (EphyEmbedShell *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
 
-  if (!priv->hosts_manager)
-    priv->hosts_manager = ephy_hosts_manager_new ();
-  return priv->hosts_manager;
+  return priv->permissions_manager;
 }
