@@ -28,6 +28,8 @@
 #include "ephy-uri-tester.h"
 
 #include "ephy-debug.h"
+#include "ephy-prefs.h"
+#include "ephy-settings.h"
 #include "ephy-uri-tester-shared.h"
 
 #include <gio/gio.h>
@@ -65,6 +67,7 @@ struct _EphyUriTester {
   GRegex *regex_frame_add;
 
   GMainLoop *load_loop;
+  int adblock_filters_to_load;
   gboolean adblock_loaded;
 #ifdef HAVE_LIBHTTPSEVERYWHERE
   gboolean https_everywhere_loaded;
@@ -213,7 +216,6 @@ static GString *
 ephy_uri_tester_fixup_regexp (const char *prefix, char *src)
 {
   GString *str;
-  int len = 0;
 
   if (!src)
     return NULL;
@@ -225,26 +227,56 @@ ephy_uri_tester_fixup_regexp (const char *prefix, char *src)
     (void)*src++;
   }
 
+  /* NOTE: The '$' is used as separator for the rule options, so rule patterns
+     cannot ever contain them. If a rule needs to match it, it uses "%24".
+     Splitting the option is done in ephy_uri_tester_add_url_pattern().
+
+     The loop below always escapes square brackets. This way there is no chance
+     that they get interpreted as a character class, and it is NOT needed to
+     escape '-' because it's only special inside a character class. */
   do {
     switch (*src) {
       case '*':
         g_string_append (str, ".*");
         break;
-      /*case '.':
-         g_string_append (str, "\\.");
-         break;*/
+      case '^':
+      /* Matches a separator character, defined as:
+       * "anything but a letter, a digit, or one of the following: _ - . %" */
+        g_string_append (str, "([^a-zA-Z\\d]|[_\\-\\.%])");
+        break;
+      case '|':
+      /* If at the end of the pattern, the match is anchored at the end. In
+       * the middle of a pattern it matches a literal vertical bar and the
+       * character must be escaped. */
+        if (src[1] == '\0')
+          g_string_append (str, "$");
+        else
+          g_string_append (str, "\\|");
+        break;
+      /* The following characters are escaped as they have a meaning in
+       * regular expressions:
+       *   - '.' matches any character.
+       *   - '+' matches the preceding pattern one or more times.
+       *   - '?' matches the preceding pattern zero or one times.
+       *   - '[' ']' are used to define a character class.
+       *   - '{' '}' are used to define a min/max quantifier.
+       *   - '(' ')' are used to defin a submatch expression.
+       *   - '\' has several uses in regexps (shortcut character classes.
+       *     matching non-printing characters, using octal/hex, octal
+       *     constants, backreferences... they must to be escaped to
+       *     match a literal backslash and prevent wrecking havoc!). */
+      case '.':
+      case '+':
       case '?':
       case '[':
       case ']':
+      case '{':
+      case '}':
+      case '(':
+      case ')':
+      case '\\':
         g_string_append_printf (str, "\\%c", *src);
         break;
-      case '|':
-      /* FIXME: We actually need to match :[0-9]+ or '/'. Sign means
-         "here could be port number or nothing". So bla.com^ will match
-         bla.com/ or bla.com:8080/ but not bla.com.au/ */
-      case '^':
-      case '+':
-        break;
       default:
         g_string_append_printf (str, "%c", *src);
         break;
@@ -252,11 +284,6 @@ ephy_uri_tester_fixup_regexp (const char *prefix, char *src)
     src++;
   } while (*src);
 
-  len = str->len;
-  /* We dont need .* in the end of url. Thats stupid */
-  if (str->str && str->str[len - 1] == '*' && str->str[len - 2] == '.')
-    g_string_erase (str, len - 2, 2);
-
   return str;
 }
 
@@ -509,13 +536,15 @@ ephy_uri_tester_parse_line (EphyUriTester *tester,
 static void
 ephy_uri_tester_adblock_loaded (EphyUriTester *tester)
 {
-  tester->adblock_loaded = TRUE;
+  if (g_atomic_int_dec_and_test (&tester->adblock_filters_to_load)) {
+    tester->adblock_loaded = TRUE;
 #ifdef HAVE_LIBHTTPSEVERYWHERE
-  if (tester->https_everywhere_loaded)
-    g_main_loop_quit (tester->load_loop);
+    if (tester->https_everywhere_loaded)
+      g_main_loop_quit (tester->load_loop);
 #else
-  g_main_loop_quit (tester->load_loop);
+    g_main_loop_quit (tester->load_loop);
 #endif
+  }
 }
 
 #ifdef HAVE_LIBHTTPSEVERYWHERE
@@ -648,50 +677,71 @@ adblock_file_monitor_changed (GFileMonitor     *monitor,
                      tester);
 }
 
+static void
+ephy_uri_tester_begin_loading_adblock_filters (EphyUriTester  *tester,
+                                               GList         **monitors)
+{
+  char **filters;
+
+  filters = g_settings_get_strv (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ADBLOCK_FILTERS);
+  tester->adblock_filters_to_load = g_strv_length (filters);
+  for (guint i = 0; filters[i]; i++) {
+    GFile *filter_file;
+
+    filter_file = ephy_uri_tester_get_adblock_filter_file (tester->adblock_data_dir, filters[i]);
+    if (!g_file_query_exists (filter_file, NULL)) {
+      GFileMonitor *monitor;
+      GError *error = NULL;
+
+      monitor = g_file_monitor_file (filter_file, G_FILE_MONITOR_WATCH_MOVES, NULL, &error);
+      if (monitor) {
+        *monitors = g_list_prepend (*monitors, monitor);
+        g_signal_connect (monitor, "changed", G_CALLBACK (adblock_file_monitor_changed), tester);
+      } else {
+        g_warning ("Failed to monitor adblock file: %s\n", error->message);
+        g_error_free (error);
+        ephy_uri_tester_adblock_loaded (tester);
+      }
+    } else {
+      g_file_read_async (filter_file, G_PRIORITY_DEFAULT_IDLE, NULL,
+                         (GAsyncReadyCallback)file_read_cb,
+                         tester);
+    }
+    g_object_unref (filter_file);
+  }
+  g_strfreev (filters);
+}
+
 static void
 ephy_uri_tester_load_sync (GTask         *task,
                            EphyUriTester *tester)
 {
   GMainContext *context;
-  GFile *filter_file;
-  GFileMonitor *monitor = NULL;
+  GList *monitors = NULL;
 
   context = g_main_context_new ();
-  tester->load_loop = g_main_loop_new (context, FALSE);
   g_main_context_push_thread_default (context);
+  tester->load_loop = g_main_loop_new (context, FALSE);
 
 #ifdef HAVE_LIBHTTPSEVERYWHERE
-  tester->https_everywhere_context = https_everywhere_context_new ();
-  https_everywhere_context_init (tester->https_everywhere_context, NULL,
-                                 (GAsyncReadyCallback)https_everywhere_context_init_cb,
-                                 tester);
+  if (!tester->https_everywhere_loaded) {
+    g_assert (tester->https_everywhere_context == NULL);
+    tester->https_everywhere_context = https_everywhere_context_new ();
+    https_everywhere_context_init (tester->https_everywhere_context, NULL,
+                                   (GAsyncReadyCallback)https_everywhere_context_init_cb,
+                                   tester);
+  }
 #endif
 
-  filter_file = ephy_uri_tester_get_adblock_filter_file (tester->adblock_data_dir);
-  if (!g_file_query_exists (filter_file, NULL)) {
-    GError *error = NULL;
-
-    monitor = g_file_monitor_file (filter_file, G_FILE_MONITOR_WATCH_MOVES, NULL, &error);
-    if (monitor) {
-       g_signal_connect (monitor, "changed", G_CALLBACK (adblock_file_monitor_changed), tester);
-    } else {
-      g_warning ("Failed to monitor adblock file: %s\n", error->message);
-      g_error_free (error);
-      ephy_uri_tester_adblock_loaded (tester);
-    }
-  } else {
-    g_file_read_async (filter_file, G_PRIORITY_DEFAULT_IDLE, NULL,
-                       (GAsyncReadyCallback)file_read_cb,
-                       tester);
-  }
-  g_object_unref (filter_file);
+  if (!tester->adblock_loaded)
+    ephy_uri_tester_begin_loading_adblock_filters (tester, &monitors);
 
   g_main_loop_run (tester->load_loop);
 
-  if (monitor)
-    g_object_unref (monitor);
+  g_list_free_full (monitors, g_object_unref);
   g_main_context_pop_thread_default (context);
   g_main_context_unref (context);
+  g_main_loop_unref (tester->load_loop);
 
   g_task_return_boolean (task, TRUE);
 }
@@ -809,9 +859,6 @@ ephy_uri_tester_finalize (GObject *object)
   g_regex_unref (tester->regex_subdocument);
   g_regex_unref (tester->regex_frame_add);
 
-  if (tester->load_loop)
-    g_main_loop_unref (tester->load_loop);
-
   G_OBJECT_CLASS (ephy_uri_tester_parent_class)->finalize (object);
 }
 
@@ -840,13 +887,50 @@ ephy_uri_tester_new (const char *adblock_data_dir)
   return EPHY_URI_TESTER (g_object_new (EPHY_TYPE_URI_TESTER, "adblock-data-dir", adblock_data_dir, NULL));
 }
 
+static void
+ephy_uri_tester_reload_adblock_filters (EphyUriTester *tester)
+{
+  g_hash_table_remove_all (tester->pattern);
+  g_hash_table_remove_all (tester->keys);
+  g_hash_table_remove_all (tester->optslist);
+  g_hash_table_remove_all (tester->urlcache);
+
+  g_hash_table_remove_all (tester->whitelisted_pattern);
+  g_hash_table_remove_all (tester->whitelisted_keys);
+  g_hash_table_remove_all (tester->whitelisted_optslist);
+  g_hash_table_remove_all (tester->whitelisted_urlcache);
+
+  tester->adblock_loaded = FALSE;
+  ephy_uri_tester_load (tester);
+}
+
+static void
+ephy_uri_tester_adblock_filters_changed_cb (GSettings     *settings,
+                                            char          *key,
+                                            EphyUriTester *tester)
+{
+  ephy_uri_tester_reload_adblock_filters (tester);
+}
+
+static void
+ephy_uri_tester_enable_adblock_changed_cb (GSettings     *settings,
+                                           char          *key,
+                                           EphyUriTester *tester)
+{
+  ephy_uri_tester_reload_adblock_filters (tester);
+}
+
 void
 ephy_uri_tester_load (EphyUriTester *tester)
 {
   GTask *task;
+  char **trash;
 
   g_return_if_fail (EPHY_IS_URI_TESTER (tester));
 
+  if (!g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ENABLE_ADBLOCK))
+    tester->adblock_loaded = TRUE;
+
   if (tester->adblock_loaded
 #ifdef HAVE_LIBHTTPSEVERYWHERE
       && tester->https_everywhere_loaded
@@ -854,7 +938,19 @@ ephy_uri_tester_load (EphyUriTester *tester)
      )
     return;
 
+  g_signal_handlers_disconnect_by_func (EPHY_SETTINGS_WEB, ephy_uri_tester_adblock_filters_changed_cb, tester);
+  g_signal_handlers_disconnect_by_func (EPHY_SETTINGS_WEB, ephy_uri_tester_enable_adblock_changed_cb, tester);
+
   task = g_task_new (tester, NULL, NULL, NULL);
   g_task_run_in_thread_sync (task, (GTaskThreadFunc)ephy_uri_tester_load_sync);
   g_object_unref (task);
+
+  g_signal_connect (EPHY_SETTINGS_WEB, "changed::" EPHY_PREFS_WEB_ADBLOCK_FILTERS,
+                    G_CALLBACK (ephy_uri_tester_adblock_filters_changed_cb), tester);
+  g_signal_connect (EPHY_SETTINGS_WEB, "changed::" EPHY_PREFS_WEB_ENABLE_ADBLOCK,
+                    G_CALLBACK (ephy_uri_tester_enable_adblock_changed_cb), tester);
+  /* GSettings never emits the changed signal until after we read the setting
+   * the first time after connecting the handler... work around this.*/
+  trash = g_settings_get_strv (EPHY_SETTINGS_WEB, "adblock-filters");
+  g_strfreev (trash);
 }
