@@ -27,7 +27,7 @@
 #include "ephy-embed-form-auth.h"
 #include "ephy-file-helpers.h"
 #include "ephy-form-auth-data.h"
-#include "ephy-hosts-manager.h"
+#include "ephy-permissions-manager.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-uri-helpers.h"
@@ -54,7 +54,7 @@ struct _EphyWebExtension {
   EphyFormAuthDataCache *form_auth_data_cache;
   GHashTable *form_auth_data_save_requests;
   EphyWebOverviewModel *overview_model;
-  EphyHostsManager *hosts_manager;
+  EphyPermissionsManager *permissions_manager;
   EphyUriTester *uri_tester;
 };
 
@@ -289,15 +289,15 @@ store_password (EphyEmbedFormAuth *form_auth)
                              username_field_value,
                              password_field_value,
                              NULL, NULL);
-  g_free (uri_str);
 
   /* Update internal caching */
   ephy_form_auth_data_cache_add (extension->form_auth_data_cache,
-                                 uri->host,
+                                 uri_str,
                                  username_field_name,
                                  password_field_name,
                                  username_field_value);
 
+  g_free (uri_str);
   g_free (username_field_name);
   g_free (username_field_value);
   g_free (password_field_name);
@@ -311,18 +311,27 @@ request_decision_on_storing (EphyEmbedFormAuth *form_auth)
   guint request_id;
   SoupURI *uri;
   WebKitDOMNode *username_node;
-  WebKitDOMDOMWindow *dom_window;
+  WebKitDOMDOMWindow *dom_window = NULL;
   GVariant *variant;
-  gchar *message;
+  char *message = NULL;
+  char *uri_string = NULL;
+  char *origin = NULL;
 
   dom_window = webkit_dom_document_get_default_view (ephy_embed_form_auth_get_owner_document (form_auth));
-  if (!dom_window) {
-    g_object_unref (form_auth);
-    return;
-  }
+  if (dom_window == NULL)
+    goto out;
+
+  uri = ephy_embed_form_auth_get_uri (form_auth);
+  if (uri == NULL)
+    goto out;
+
+  uri_string = soup_uri_to_string (uri, FALSE);
+  origin = ephy_uri_to_security_origin (uri_string);
+  if (origin == NULL)
+    goto out;
 
   request_id = form_auth_data_save_request_new_id ();
-  uri = ephy_embed_form_auth_get_uri (form_auth);
+
   username_node = ephy_embed_form_auth_get_username_node (form_auth);
   if (username_node)
     g_object_get (username_node, "value", &username_field_value, NULL);
@@ -330,7 +339,7 @@ request_decision_on_storing (EphyEmbedFormAuth *form_auth)
   variant = g_variant_new ("(utss)",
                            request_id,
                            ephy_embed_form_auth_get_page_id (form_auth),
-                           uri ? uri->host : "",
+                           origin,
                            username_field_value ? username_field_value : "");
   g_free (username_field_value);
 
@@ -345,9 +354,17 @@ request_decision_on_storing (EphyEmbedFormAuth *form_auth)
     g_warning ("Error sending formAuthData message");
   }
 
-  g_object_unref (dom_window);
-  g_free (message);
-  g_object_unref (form_auth);
+out:
+  if (dom_window != NULL)
+    g_object_unref (dom_window);
+  if (form_auth != NULL)
+    g_object_unref (form_auth);
+  if (message != NULL)
+    g_free (message);
+  if (uri_string != NULL)
+    g_free (uri_string);
+  if (origin != NULL)
+    g_free (origin);
 }
 
 static void
@@ -357,22 +374,26 @@ should_store_cb (const char *username,
 {
   EphyEmbedFormAuth *form_auth = EPHY_EMBED_FORM_AUTH (user_data);
   EphyWebExtension *web_extension;
-  EphyHostPermission permission;
+  EphyPermission permission;
   SoupURI *uri;
   char *uri_string;
   char *password_field_value = NULL;
+  char *origin = NULL;
 
   uri = ephy_embed_form_auth_get_uri (form_auth);
   uri_string = soup_uri_to_string (uri, FALSE);
-  if (!uri_string)
+  if (uri_string == NULL)
     return;
+  origin = ephy_uri_to_security_origin (uri_string);
+  if (origin == NULL)
+    goto out;
 
   web_extension = ephy_web_extension_get ();
-  permission = ephy_hosts_manager_get_permission_for_address (web_extension->hosts_manager,
-                                                              EPHY_HOST_PERMISSION_TYPE_SAVE_PASSWORD,
-                                                              uri_string);
+  permission = ephy_permissions_manager_get_permission (web_extension->permissions_manager,
+                                                        EPHY_PERMISSION_TYPE_SAVE_PASSWORD,
+                                                        origin);
 
-  if (permission == EPHY_HOST_PERMISSION_DENY) {
+  if (permission == EPHY_PERMISSION_DENY) {
     LOG ("User/password storage permission previously denied. Not asking about storing.");
     goto out;
   }
@@ -395,7 +416,7 @@ should_store_cb (const char *username,
     if (g_strcmp0 (username, username_field_value) == 0 &&
         g_strcmp0 (password, password_field_value) == 0) {
       LOG ("User/password already stored. Not asking about storing.");
-    } else if (permission == EPHY_HOST_PERMISSION_ALLOW) {
+    } else if (permission == EPHY_PERMISSION_PERMIT) {
       LOG ("User/password not yet stored. Storing.");
       store_password (form_auth);
     } else {
@@ -404,7 +425,6 @@ should_store_cb (const char *username,
     }
 
     g_free (username_field_value);
-
   } else {
     LOG ("No result on query; asking whether we should store.");
     request_decision_on_storing (g_object_ref (form_auth));
@@ -413,6 +433,8 @@ should_store_cb (const char *username,
 out:
   if (password_field_value)
     g_free (password_field_value);
+  if (origin != NULL)
+    g_free (origin);
   g_free (uri_string);
 }
 
@@ -536,14 +558,15 @@ pre_fill_form (EphyEmbedFormAuth *form_auth)
     return;
 
   extension = ephy_web_extension_get ();
-  form_auth_data_list = ephy_form_auth_data_cache_get_list (extension->form_auth_data_cache, uri->host);
+  uri_str = soup_uri_to_string (uri, FALSE);
+  form_auth_data_list = ephy_form_auth_data_cache_get_list (extension->form_auth_data_cache, uri_str);
   l = g_slist_find_custom (form_auth_data_list, form_auth, (GCompareFunc)ephy_form_auth_data_compare);
-  if (!l)
+  if (!l) {
+    g_free (uri_str);
     return;
+  }
 
   form_data = (EphyFormAuthData *)l->data;
-  uri_str = soup_uri_to_string (uri, FALSE);
-
   username_node = ephy_embed_form_auth_get_username_node (form_auth);
   if (username_node)
     g_object_get (username_node, "value", &username, NULL);
@@ -1044,7 +1067,7 @@ sensitive_form_focused_cb (WebKitDOMHTMLFormElement *form,
    * unless it's clearly not. Insecure forms on insecure pages will be detected
    * in the UI process. Note that basically no websites should actually be dumb
    * enough to trip this, but no doubt they exist somewhere.... */
-  insecure_action = action == NULL && g_str_has_prefix (action, "http://");
+  insecure_action = action != NULL && g_str_has_prefix (action, "http://");
 
   variant = g_variant_new ("(tb)",
                            webkit_web_page_get_id (web_page),
@@ -1099,8 +1122,7 @@ web_page_form_controls_associated (WebKitWebPage    *web_page,
     if (ephy_web_dom_utils_find_form_auth_elements (form, &username_node, &password_node)) {
       EphyEmbedFormAuth *form_auth;
       GSList *auth_data_list;
-      const char *uri_string;
-      SoupURI *uri;
+      const char *uri;
 
       LOG ("Hooking and pre-filling a form");
 
@@ -1116,12 +1138,8 @@ web_page_form_controls_associated (WebKitWebPage    *web_page,
       }
 
       /* Plug in the user autocomplete */
-      uri_string = webkit_web_page_get_uri (web_page);
-      uri = soup_uri_new (uri_string);
-
-      auth_data_list = ephy_form_auth_data_cache_get_list (extension->form_auth_data_cache, uri->host);
-
-      soup_uri_free (uri);
+      uri = webkit_web_page_get_uri (web_page);
+      auth_data_list = ephy_form_auth_data_cache_get_list (extension->form_auth_data_cache, uri);
 
       if (auth_data_list && auth_data_list->next && username_node) {
         LOG ("More than 1 password saved, hooking menu for choosing which on focus");
@@ -1452,7 +1470,7 @@ ephy_web_extension_dispose (GObject *object)
 
   g_clear_object (&extension->uri_tester);
   g_clear_object (&extension->overview_model);
-  g_clear_object (&extension->hosts_manager);
+  g_clear_object (&extension->permissions_manager);
 
   g_clear_pointer (&extension->form_auth_data_cache,
                    ephy_form_auth_data_cache_free);
@@ -1568,7 +1586,7 @@ ephy_web_extension_initialize (EphyWebExtension   *extension,
   if (!is_private_profile)
     extension->form_auth_data_cache = ephy_form_auth_data_cache_new ();
 
-  extension->hosts_manager = ephy_hosts_manager_new ();
+  extension->permissions_manager = ephy_permissions_manager_new ();
 
   g_signal_connect_swapped (extension->extension, "page-created",
                             G_CALLBACK (ephy_web_extension_page_created_cb),
