@@ -1,30 +1,32 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set sw=2 ts=2 sts=2 et: */
 /*
  *  Copyright © 2010 Igalia S.L.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ *  This file is part of Epiphany.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Epiphany is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Epiphany is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef EPHY_SETTINGS_H
-#define EPHY_SETTINGS_H
+#pragma once
 
 #include <glib.h>
 #include <gio/gio.h>
 
 #include "ephy-prefs.h"
 
+G_BEGIN_DECLS
+
 #define EPHY_SETTINGS_MAIN      ephy_settings_get (EPHY_PREFS_SCHEMA)
 #define EPHY_SETTINGS_UI        ephy_settings_get (EPHY_PREFS_UI_SCHEMA)
 #define EPHY_SETTINGS_WEB       ephy_settings_get (EPHY_PREFS_WEB_SCHEMA)
@@ -33,6 +35,9 @@
 
 GSettings *ephy_settings_get (const char *schema);
 
+void ephy_settings_ensure_schema_for_path (const char *schema,
+                                           const char *path);
+
 void ephy_settings_shutdown (void);
 
-#endif /* EPHY_SETTINGS_H */
+G_END_DECLS
