@@ -62,35 +62,49 @@ static GParamSpec *obj_properties[LAST_PROP];
 static GtkWidget *create_bookmark_row (gpointer item, gpointer user_data);
 
 static void
-bookmark_tag_added_cb (EphyBookmark         *bookmark,
-                       EphyBookmarksPopover *popover)
+ephy_bookmarks_popover_bookmark_tag_added_cb (EphyBookmarksPopover *self,
+                                              EphyBookmark         *bookmark,
+                                              const char           *tag,
+                                              EphyBookmarksManager *manager)
 {
   g_assert (EPHY_IS_BOOKMARK (bookmark));
-  g_assert (EPHY_IS_BOOKMARKS_POPOVER (popover));
+  g_assert (EPHY_IS_BOOKMARKS_POPOVER (self));
 
   /* If the bookmark no longer has 0 tags, we remove it from the tags list box */
   if (g_sequence_get_length (ephy_bookmark_get_tags (bookmark)) == 1) {
     GList *children;
     GList *l;
+    const char *visible_stack_child;
 
-    children = gtk_container_get_children (GTK_CONTAINER (popover->tags_list_box));
+    children = gtk_container_get_children (GTK_CONTAINER (self->tags_list_box));
     for (l = children; l != NULL; l = l->next) {
       const char *url;
 
       url = g_object_get_data (G_OBJECT (l->data), "url");
       if (g_strcmp0 (ephy_bookmark_get_url (bookmark), url) == 0)
-        gtk_container_remove (GTK_CONTAINER (popover->tags_list_box), GTK_WIDGET (l->data));
+        gtk_container_remove (GTK_CONTAINER (self->tags_list_box), GTK_WIDGET (l->data));
+    }
+
+    /* If we are on the tag detail list box, then the user has toggled the state
+     * of the tag widget multiple times. The first time the bookmark was removed
+     * from the list box. Now we have to add it back. */
+    visible_stack_child = gtk_stack_get_visible_child_name (GTK_STACK (self->toplevel_stack));
+    if (g_strcmp0 (visible_stack_child, "tag_detail") == 0) {
+      GtkWidget *row;
+      row = create_bookmark_row (bookmark, self);
+      gtk_container_add (GTK_CONTAINER (self->tag_detail_list_box), row);
     }
   }
 }
 
 static void
-bookmark_tag_removed_cb (EphyBookmark         *bookmark,
-                         const char           *tag,
-                         EphyBookmarksPopover *popover)
+ephy_bookmarks_popover_bookmark_tag_removed_cb (EphyBookmarksPopover *self,
+                                                EphyBookmark         *bookmark,
+                                                const char           *tag,
+                                                EphyBookmarksManager *manager)
 {
   g_assert (EPHY_IS_BOOKMARK (bookmark));
-  g_assert (EPHY_IS_BOOKMARKS_POPOVER (popover));
+  g_assert (EPHY_IS_BOOKMARKS_POPOVER (self));
 
   /* If the bookmark has 0 tags after removing one, we add it to the tags list
    * box */
@@ -102,7 +116,7 @@ bookmark_tag_removed_cb (EphyBookmark         *bookmark,
     gboolean exists;
 
     exists = FALSE;
-    children = gtk_container_get_children (GTK_CONTAINER (popover->tags_list_box));
+    children = gtk_container_get_children (GTK_CONTAINER (self->tags_list_box));
     for (l = children; l != NULL; l = l->next) {
       const char *url;
 
@@ -114,32 +128,29 @@ bookmark_tag_removed_cb (EphyBookmark         *bookmark,
     }
 
     if (!exists) {
-      /* Disconnect the previous handler on the instance as create bookmark row
-       * connects another one */
-      g_signal_handlers_disconnect_by_data (bookmark, popover);
-      row = create_bookmark_row (bookmark, popover);
-      gtk_container_add (GTK_CONTAINER (popover->tags_list_box), row);
+      row = create_bookmark_row (bookmark, self);
+      gtk_container_add (GTK_CONTAINER (self->tags_list_box), row);
     }
 
-    /* If we are on tag detail list box, we remove the tag from it to reflect
-     * the changes */
-    visible_stack_child = gtk_stack_get_visible_child_name (GTK_STACK (popover->toplevel_stack));
+    /* If we are on tag detail list box, we remove the bookmark from it to
+     * reflect the changes */
+    visible_stack_child = gtk_stack_get_visible_child_name (GTK_STACK (self->toplevel_stack));
     if (g_strcmp0 (visible_stack_child, "tag_detail") == 0) {
-      children = gtk_container_get_children (GTK_CONTAINER (popover->tag_detail_list_box));
+      children = gtk_container_get_children (GTK_CONTAINER (self->tag_detail_list_box));
       for (l = children; l != NULL; l = l->next) {
         const char *url;
 
         url = g_object_get_data (G_OBJECT (l->data), "url");
         if (g_strcmp0 (ephy_bookmark_get_url (bookmark), url) == 0) {
-          gtk_container_remove (GTK_CONTAINER (popover->tag_detail_list_box), GTK_WIDGET (l->data));
+          gtk_container_remove (GTK_CONTAINER (self->tag_detail_list_box), GTK_WIDGET (l->data));
 
           /* If we removed a tag's last bookmark while on the tags detail box,
            * we switch back to the tags list stack */
-          if (g_sequence_is_empty (ephy_bookmarks_manager_get_bookmarks_with_tag (popover->manager, tag))) {
+          if (g_sequence_is_empty (ephy_bookmarks_manager_get_bookmarks_with_tag (self->manager, tag))) {
             GActionGroup *group;
             GAction *action;
 
-            group = gtk_widget_get_action_group (GTK_WIDGET (popover), "popover");
+            group = gtk_widget_get_action_group (GTK_WIDGET (self), "popover");
             action = g_action_map_lookup_action (G_ACTION_MAP (group), "tag-detail-back");
             g_action_activate (action, NULL);
           }
@@ -169,13 +180,6 @@ create_bookmark_row (gpointer item,
                           g_strdup (ephy_bookmark_get_url (bookmark)),
                           (GDestroyNotify)g_free);
 
-  g_signal_connect_object (bookmark, "tag-added",
-                           G_CALLBACK (bookmark_tag_added_cb),
-                           user_data, 0);
-  g_signal_connect_object (bookmark, "tag-removed",
-                           G_CALLBACK (bookmark_tag_removed_cb),
-                           user_data, 0);
-
   return row;
 }
 
@@ -199,13 +203,10 @@ create_tag_row (const char *tag)
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_widget_set_halign (box, GTK_ALIGN_START);
 
-  if (g_strcmp0 (tag, "Favorites") == 0)
-  {
+  if (g_strcmp0 (tag, "Favorites") == 0) {
     image = gtk_image_new_from_icon_name ("user-bookmarks-symbolic", GTK_ICON_SIZE_MENU);
     label = gtk_label_new (_("Favorites"));
-  }
-  else
-  {
+  } else {
     image = gtk_image_new_from_icon_name ("ephy-bookmark-tag-symbolic", GTK_ICON_SIZE_MENU);
     label = gtk_label_new (tag);
   }
@@ -237,6 +238,9 @@ ephy_bookmarks_popover_bookmark_added_cb (EphyBookmarksPopover *self,
     row = create_bookmark_row (bookmark, self);
     gtk_container_add (GTK_CONTAINER (self->tags_list_box), row);
   }
+
+  if (strcmp (gtk_stack_get_visible_child_name (GTK_STACK (self->toplevel_stack)), "empty-state") == 0)
+    gtk_stack_set_visible_child_name (GTK_STACK (self->toplevel_stack), "default");
 }
 
 static void
@@ -270,6 +274,9 @@ ephy_bookmarks_popover_bookmark_removed_cb (EphyBookmarksPopover *self,
   }
 
   gtk_container_remove (GTK_CONTAINER (self->tags_list_box), row);
+
+  if (g_list_model_get_n_items (G_LIST_MODEL (self->list_model)) == 0)
+    gtk_stack_set_visible_child_name (GTK_STACK (self->toplevel_stack), "empty-state");
 }
 
 static void
@@ -502,6 +509,9 @@ ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
                            create_bookmark_row,
                            self, NULL);
 
+  if (g_list_model_get_n_items (G_LIST_MODEL (self->list_model)) == 0)
+    gtk_stack_set_visible_child_name (GTK_STACK (self->toplevel_stack), "empty-state");
+
   gtk_list_box_set_sort_func (GTK_LIST_BOX (self->tags_list_box),
                               (GtkListBoxSortFunc)tags_list_box_sort_func,
                               NULL, NULL);
@@ -546,6 +556,12 @@ ephy_bookmarks_popover_init (EphyBookmarksPopover *self)
   g_signal_connect_object (self->manager, "tag-deleted",
                            G_CALLBACK (ephy_bookmarks_popover_tag_deleted_cb),
                            self, G_CONNECT_SWAPPED);
+  g_signal_connect_object (self->manager, "bookmark-tag-added",
+                           G_CALLBACK (ephy_bookmarks_popover_bookmark_tag_added_cb),
+                           self, G_CONNECT_SWAPPED);
+  g_signal_connect_object (self->manager, "bookmark-tag-removed",
+                           G_CALLBACK (ephy_bookmarks_popover_bookmark_tag_removed_cb),
+                           self, G_CONNECT_SWAPPED);
 
   g_signal_connect_object (self->bookmarks_list_box, "row-activated",
                            G_CALLBACK (ephy_bookmarks_popover_list_box_row_activated_cb),
