@@ -189,10 +189,11 @@ homepage_url_changed (GSettings  *settings,
                       const char *key,
                       GtkWidget  *button)
 {
-  const char *setting;
+  char *setting;
 
   setting = g_settings_get_string (settings, key);
   gtk_widget_set_visible (button, setting && setting[0]);
+  g_free (setting);
 }
 
 typedef enum {
@@ -577,8 +578,9 @@ ephy_header_bar_change_combined_stop_reload_state (GSimpleAction *action,
     gtk_widget_set_tooltip_text (header_bar->combined_stop_reload_button, _(REFRESH_BUTTON_TOOLTIP));
   }
 
-  gtk_button_set_image (GTK_BUTTON (header_bar->combined_stop_reload_button),
-                        image);
+  gtk_button_set_image (GTK_BUTTON (header_bar->combined_stop_reload_button), image);
+
+  g_simple_action_set_state (action, loading);
 }
 
 static void
@@ -717,14 +719,12 @@ ephy_header_bar_constructed (GObject *object)
   }
 
   /* Title widget (location entry or title box) */
-  if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION) {
+  if (ephy_embed_shell_get_mode (embed_shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_title_box_new ());
-  } else {
+  else
     header_bar->title_widget = EPHY_TITLE_WIDGET (ephy_location_entry_new ());
-    gtk_widget_set_margin_start (GTK_WIDGET (header_bar->title_widget), 30);
-    gtk_widget_set_margin_end (GTK_WIDGET (header_bar->title_widget), 30);
-  }
-
+  gtk_widget_set_margin_start (GTK_WIDGET (header_bar->title_widget), 54);
+  gtk_widget_set_margin_end (GTK_WIDGET (header_bar->title_widget), 54);
   gtk_header_bar_set_custom_title (GTK_HEADER_BAR (header_bar), GTK_WIDGET (header_bar->title_widget));
   gtk_widget_show (GTK_WIDGET (header_bar->title_widget));
 
