@@ -24,7 +24,6 @@
 
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
-#include "ephy-private.h"
 #include "ephy-profile-utils.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
@@ -34,10 +33,12 @@
 
 #include <errno.h>
 #include <glib/gi18n.h>
+#include <glib-unix.h>
 #include <gtk/gtk.h>
 #include <libnotify/notify.h>
 #include <libxml/xmlreader.h>
 #include <libxml/xmlversion.h>
+#include <signal.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -56,6 +57,31 @@ static gboolean application_mode = FALSE;
 static char *desktop_file_basename = NULL;
 static char *profile_directory = NULL;
 
+static EphyShell *ephy_shell = NULL;
+static int shutdown_signum = 0;
+
+static gboolean
+handle_shutdown_signal (gpointer user_data)
+{
+  shutdown_signum = GPOINTER_TO_INT (user_data);
+
+  /* Note that this function executes on the main loop AFTER the signal handler
+   * has returned, so we don't have to worry about async signal safety.
+   */
+  g_assert (ephy_shell != NULL);
+  ephy_shell_try_quit (ephy_shell);
+
+  /* Goals:
+   *
+   * (1) Shutdown safely and cleanly if signal is received once.
+   * (2) Shutdown unsafely but immediately if signal is received twice.
+   * (3) Always re-raise the signal so the parent process knows what happened.
+   *
+   * Removing this source is required by goals (2) and (3).
+   */
+  return G_SOURCE_REMOVE;
+}
+
 static gboolean
 application_mode_cb (const gchar *option_name,
                      const gchar *value,
@@ -141,27 +167,6 @@ get_startup_id (void)
   return retval;
 }
 
-static void
-show_error_message (GError **error)
-{
-  GtkWidget *dialog;
-
-  /* FIXME better texts!!! */
-  dialog = gtk_message_dialog_new (NULL,
-                                   GTK_DIALOG_MODAL,
-                                   GTK_MESSAGE_ERROR,
-                                   GTK_BUTTONS_CLOSE,
-                                   _("Could not start Web"));
-  gtk_message_dialog_format_secondary_text
-    (GTK_MESSAGE_DIALOG (dialog),
-    _("Startup failed because of the following error:\n%s"),
-    (*error)->message);
-
-  g_clear_error (error);
-
-  gtk_dialog_run (GTK_DIALOG (dialog));
-}
-
 static EphyStartupFlags
 get_startup_flags (void)
 {
@@ -187,7 +192,6 @@ main (int   argc,
   EphyShellStartupContext *ctx;
   EphyStartupFlags startup_flags;
   EphyEmbedShellMode mode;
-  EphyShell *ephy_shell;
   int status;
   EphyFileHelpersFlags flags;
   GDesktopAppInfo *desktop_info = NULL;
@@ -320,7 +324,7 @@ main (int   argc,
   }
 
   if (incognito_mode && profile_directory == NULL)
-    profile_directory = g_strdup (ephy_dot_dir ());
+    profile_directory = ephy_default_dot_dir ();
 
   /* Start our services */
   flags = !application_mode ? EPHY_FILE_HELPERS_ENSURE_EXISTS : 0;
@@ -332,10 +336,8 @@ main (int   argc,
   if (profile_directory && !incognito_mode)
     flags |= EPHY_FILE_HELPERS_KEEP_DIR;
 
-  if (!ephy_file_helpers_init (profile_directory, flags,
-                               &error)) {
-    show_error_message (&error);
-    exit (1);
+  if (!ephy_file_helpers_init (profile_directory, flags, &error)) {
+    g_error ("Fatal initialization error: %s", error->message);
   }
 
   /* Run the migration in all cases, except when running a private
@@ -378,6 +380,19 @@ main (int   argc,
     exit (0);
   }
 
+  /* Setup relocatable schemas */
+  if (application_mode) {
+    char *web_app_name;
+    char *path;
+
+    web_app_name = g_path_get_basename (profile_directory);
+    path = g_build_path ("/", "/org/gnome/epiphany/web-apps/", web_app_name, "state/", NULL);
+    ephy_settings_ensure_schema_for_path (EPHY_PREFS_STATE_SCHEMA, path);
+    g_free (web_app_name);
+    g_free (path);
+  } else
+    ephy_settings_ensure_schema_for_path (EPHY_PREFS_STATE_SCHEMA, "/org/gnome/epiphany/state/");
+
   startup_flags = get_startup_flags ();
 
   /* Now create the shell */
@@ -423,6 +438,10 @@ main (int   argc,
   g_strfreev (arguments);
   ephy_shell = ephy_shell_get_default ();
   ephy_shell_set_startup_context (ephy_shell, ctx);
+
+  g_unix_signal_add (SIGINT, (GSourceFunc)handle_shutdown_signal, GINT_TO_POINTER (SIGINT));
+  g_unix_signal_add (SIGTERM, (GSourceFunc)handle_shutdown_signal, GINT_TO_POINTER (SIGTERM));
+
   status = g_application_run (G_APPLICATION (ephy_shell), argc, argv);
 
   /* Shutdown */
@@ -438,5 +457,8 @@ main (int   argc,
   ephy_file_helpers_shutdown ();
   xmlCleanupParser ();
 
+  if (shutdown_signum != 0)
+    raise (shutdown_signum);
+
   return status;
 }
