@@ -30,7 +30,7 @@
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
 #include "ephy-header-bar.h"
-#include "ephy-history-window.h"
+#include "ephy-history-dialog.h"
 #include "ephy-lockdown.h"
 #include "ephy-prefs.h"
 #include "ephy-session.h"
@@ -62,7 +62,7 @@ struct _EphyShell {
   GObject *lockdown;
   EphyBookmarksManager *bookmarks_manager;
   GNetworkMonitor *network_monitor;
-  GtkWidget *history_window;
+  GtkWidget *history_dialog;
   GObject *prefs_dialog;
   EphyShellStartupContext *local_startup_context;
   EphyShellStartupContext *remote_startup_context;
@@ -129,15 +129,13 @@ ephy_shell_startup_context_free (EphyShellStartupContext *ctx)
 static void
 ephy_shell_startup_continue (EphyShell *shell, EphyShellStartupContext *ctx)
 {
-  EphySession *session;
+  EphySession *session = ephy_shell_get_session (shell);
 
-  session = ephy_shell_get_session (shell);
-  g_assert (session != NULL);
-
-  if (ctx->session_filename != NULL)
+  if (ctx->session_filename != NULL) {
+    g_assert (session != NULL);
     ephy_session_load (session, (const char *)ctx->session_filename,
                        ctx->user_time, NULL, NULL, NULL);
-  else if (ctx->arguments != NULL) {
+  } else if (ctx->arguments || !session) {
     /* Don't queue any window openings if no extra arguments given, */
     /* since session autoresume will open one for us. */
     ephy_shell_open_uris (shell, (const char **)ctx->arguments,
@@ -353,7 +351,7 @@ ephy_shell_startup (GApplication *application)
 
   builder = gtk_builder_new ();
   gtk_builder_add_from_resource (builder,
-                                 "/org/gnome/epiphany/epiphany-application-menu.ui",
+                                 "/org/gnome/epiphany/gtk/application-menu.ui",
                                  NULL);
 
   mode = ephy_embed_shell_get_mode (embed_shell);
@@ -382,10 +380,6 @@ ephy_shell_startup (GApplication *application)
                       G_CALLBACK (sync_tokens_load_finished_cb), NULL);
 #endif
 
-    builder = gtk_builder_new ();
-    gtk_builder_add_from_resource (builder,
-                                   "/org/gnome/epiphany/epiphany-application-menu.ui",
-                                   NULL);
     gtk_application_set_app_menu (GTK_APPLICATION (application),
                                   G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
   } else {
@@ -423,8 +417,10 @@ ephy_shell_activate (GApplication *application)
    * can be invalidated by another remote instance.
    */
   if (shell->remote_startup_context == NULL) {
-    if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell)) != EPHY_EMBED_SHELL_MODE_APPLICATION) {
-      ephy_session_resume (ephy_shell_get_session (shell),
+    EphySession *session = ephy_shell_get_session (shell);
+
+    if (session) {
+      ephy_session_resume (session,
                            shell->local_startup_context->user_time,
                            NULL, session_load_cb, shell->local_startup_context);
     } else
@@ -647,7 +643,7 @@ ephy_shell_dispose (GObject *object)
 
   g_clear_object (&shell->session);
   g_clear_object (&shell->lockdown);
-  g_clear_pointer (&shell->history_window, gtk_widget_destroy);
+  g_clear_pointer (&shell->history_dialog, gtk_widget_destroy);
   g_clear_object (&shell->prefs_dialog);
   g_clear_object (&shell->network_monitor);
 #ifdef ENABLE_SYNC
@@ -791,8 +787,14 @@ ephy_shell_new_tab (EphyShell      *shell,
 EphySession *
 ephy_shell_get_session (EphyShell *shell)
 {
+  EphyEmbedShellMode mode;
+
   g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
 
+  mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (shell));
+  if (mode ==  EPHY_EMBED_SHELL_MODE_APPLICATION || mode == EPHY_EMBED_SHELL_MODE_INCOGNITO)
+    return NULL;
+
   if (shell->session == NULL)
     shell->session = g_object_new (EPHY_TYPE_SESSION, NULL);
 
@@ -851,29 +853,29 @@ ephy_shell_get_net_monitor (EphyShell *shell)
 }
 
 /**
- * ephy_shell_get_history_window:
+ * ephy_shell_get_history_dialog:
  *
  * Return value: (transfer none):
  **/
 GtkWidget *
-ephy_shell_get_history_window (EphyShell *shell)
+ephy_shell_get_history_dialog (EphyShell *shell)
 {
   EphyEmbedShell *embed_shell;
   EphyHistoryService *service;
 
   embed_shell = ephy_embed_shell_get_default ();
 
-  if (shell->history_window == NULL) {
+  if (shell->history_dialog == NULL) {
     service = EPHY_HISTORY_SERVICE
                 (ephy_embed_shell_get_global_history_service (embed_shell));
-    shell->history_window = ephy_history_window_new (service);
-    g_signal_connect (shell->history_window,
+    shell->history_dialog = ephy_history_dialog_new (service);
+    g_signal_connect (shell->history_dialog,
                       "destroy",
                       G_CALLBACK (gtk_widget_destroyed),
-                      &shell->history_window);
+                      &shell->history_dialog);
   }
 
-  return shell->history_window;
+  return shell->history_dialog;
 }
 
 /**
@@ -945,10 +947,12 @@ ephy_shell_close_all_windows (EphyShell *shell)
 {
   GList *windows;
   gboolean retval = TRUE;
+  EphySession *session = ephy_shell_get_session (shell);
 
   g_return_val_if_fail (EPHY_IS_SHELL (shell), FALSE);
 
-  ephy_session_close (ephy_shell_get_session (shell));
+  if (session)
+    ephy_session_close (session);
 
   windows = gtk_application_get_windows (GTK_APPLICATION (shell));
   while (windows) {
@@ -995,10 +999,11 @@ open_uris_data_new (EphyShell       *shell,
   gboolean new_windows_in_tabs;
   gboolean fullscreen_lockdown;
   gboolean have_uris;
+  EphySession *session = ephy_shell_get_session (shell);
 
   data = g_slice_new0 (OpenURIsData);
   data->shell = shell;
-  data->session = g_object_ref (ephy_shell_get_session (shell));
+  data->session = session ? g_object_ref (session) : NULL;
   data->uris = g_strdupv ((char **)uris);
   data->user_time = user_time;
 
@@ -1007,7 +1012,7 @@ open_uris_data_new (EphyShell       *shell,
   fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
                                                 EPHY_PREFS_LOCKDOWN_FULLSCREEN);
 
-  have_uris = !(g_strv_length ((char **)uris) == 1 && g_str_equal (uris[0], ""));
+  have_uris = uris && !(g_strv_length ((char **)uris) == 1 && g_str_equal (uris[0], ""));
 
   if (startup_flags & EPHY_STARTUP_NEW_WINDOW && !fullscreen_lockdown) {
     data->window = ephy_window_new ();
@@ -1028,7 +1033,7 @@ static void
 open_uris_data_free (OpenURIsData *data)
 {
   g_application_release (G_APPLICATION (data->shell));
-  g_object_unref (data->session);
+  g_clear_object (&data->session);
   g_strfreev (data->uris);
 
   g_slice_free (OpenURIsData, data);
@@ -1067,8 +1072,8 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
                                      data->user_time);
   }
 
-  url = data->uris[data->current_uri];
-  if (url[0] != '\0') {
+  url = data->uris ? data->uris[data->current_uri] : NULL;
+  if (url && url[0] != '\0') {
     ephy_web_view_load_url (ephy_embed_get_web_view (embed), url);
 
     /* When reusing an empty tab, the focus is in the location entry */
@@ -1091,7 +1096,7 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
   data->current_uri++;
   data->previous_embed = embed;
 
-  return data->uris[data->current_uri] != NULL;
+  return data->uris && data->uris[data->current_uri] != NULL;
 }
 
 static void
