@@ -28,6 +28,7 @@
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-utils.h"
+#include "ephy-encodings.h"
 #include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
@@ -36,6 +37,8 @@
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
+#include "ephy-string.h"
+#include "ephy-uri-tester-shared.h"
 #include "clear-data-dialog.h"
 #include "cookies-dialog.h"
 #include "languages.h"
@@ -87,7 +90,7 @@ struct _PrefsDialog {
   GtkWidget *css_checkbox;
   GtkWidget *css_edit_button;
 
-  /* privacy */
+  /* stored data */
   GtkWidget *always;
   GtkWidget *no_third_party;
   GtkWidget *never;
@@ -96,6 +99,7 @@ struct _PrefsDialog {
   GtkWidget *clear_personal_data_button;
 
   /* language */
+  GtkWidget *default_encoding_combo;
   GtkTreeView *lang_treeview;
   GtkWidget *lang_add_button;
   GtkWidget *lang_remove_button;
@@ -143,6 +147,12 @@ typedef struct {
 } FxACallbackData;
 #endif
 
+enum {
+  COL_TITLE_ELIDED,
+  COL_ENCODING,
+  NUM_COLS
+};
+
 enum {
   SEARCH_ENGINE_COL_NAME,
   SEARCH_ENGINE_COL_STOCK_URL,
@@ -466,6 +476,9 @@ out:
 static void
 setup_fxa_sign_in_view (PrefsDialog *dialog)
 {
+  EphyEmbedShell *shell;
+  WebKitWebContext *embed_context;
+  WebKitWebContext *sync_context;
   const char *js = "\"use strict\";"
                    "function handleAccountsCommand(evt) {"
                    "  let j = {type: evt.type, detail: evt.detail};"
@@ -486,10 +499,18 @@ setup_fxa_sign_in_view (PrefsDialog *dialog)
   webkit_user_content_manager_register_script_message_handler (dialog->fxa_manager,
                                                                "accountsCommandHandler");
 
+  shell = ephy_embed_shell_get_default ();
+  embed_context = ephy_embed_shell_get_web_context (shell);
+
+  sync_context = webkit_web_context_new ();
+  webkit_web_context_set_preferred_languages (sync_context,
+                                              g_object_get_data (G_OBJECT (embed_context), "preferred-languages"));
   dialog->fxa_web_view = WEBKIT_WEB_VIEW (g_object_new (WEBKIT_TYPE_WEB_VIEW,
                                                         "user-content-manager", dialog->fxa_manager,
                                                         "settings", ephy_embed_prefs_get_settings (),
+                                                        "web-context", sync_context,
                                                         NULL));
+  g_object_unref (sync_context);
 
   gtk_widget_set_visible (GTK_WIDGET (dialog->fxa_web_view), TRUE);
   gtk_widget_set_size_request (GTK_WIDGET (dialog->fxa_web_view), 450, 450);
@@ -569,7 +590,7 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   object_class->finalize = prefs_dialog_finalize;
 
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/org/gnome/epiphany/prefs-dialog.ui");
+                                               "/org/gnome/epiphany/gtk/prefs-dialog.ui");
   /* general */
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, new_tab_homepage_radiobutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, blank_homepage_radiobutton);
@@ -593,7 +614,7 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, css_checkbox);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, css_edit_button);
 
-  /* privacy */
+  /* stored data */
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, always);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, no_third_party);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, never);
@@ -602,6 +623,7 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, clear_personal_data_button);
 
   /* language */
+  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, default_encoding_combo);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, lang_treeview);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, lang_add_button);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, lang_remove_button);
@@ -701,6 +723,67 @@ combo_set_mapping (const GValue       *value,
   return variant;
 }
 
+static void
+create_node_combo (PrefsDialog   *dialog,
+                   EphyEncodings *encodings,
+                   const char    *default_value)
+{
+  GtkComboBox *combo;
+  GtkCellRenderer *renderer;
+  GtkListStore *store;
+  GList *all_encodings, *p;
+  char *code;
+
+  code = g_settings_get_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ENCODING);
+  if (code == NULL || ephy_encodings_get_encoding (encodings, code, FALSE) == NULL)
+  {
+    /* safe default */
+    g_settings_set_string (EPHY_SETTINGS_WEB,
+                           EPHY_PREFS_WEB_DEFAULT_ENCODING,
+                           default_value);
+  }
+  g_free (code);
+
+  combo = GTK_COMBO_BOX (dialog->default_encoding_combo);
+
+  store = gtk_list_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_STRING);
+  all_encodings = ephy_encodings_get_all (encodings);
+  for (p = all_encodings; p; p = p->next) {
+    GtkTreeIter iter;
+    EphyEncoding *encoding = EPHY_ENCODING (p->data);
+
+    gtk_list_store_append (store, &iter);
+    gtk_list_store_set (store, &iter,
+                        COL_TITLE_ELIDED,
+                        ephy_encoding_get_title_elided (encoding),
+                        -1);
+    gtk_list_store_set (store, &iter,
+                        COL_ENCODING,
+                        ephy_encoding_get_encoding (encoding),
+                        -1);
+  }
+  g_list_free (all_encodings);
+
+  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), COL_TITLE_ELIDED,
+                                        GTK_SORT_ASCENDING);
+  gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
+
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
+  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
+                                  "text", COL_TITLE_ELIDED,
+                                  NULL);
+
+  g_settings_bind_with_mapping (EPHY_SETTINGS_WEB,
+                                EPHY_PREFS_WEB_DEFAULT_ENCODING,
+                                combo, "active",
+                                G_SETTINGS_BIND_DEFAULT,
+                                combo_get_mapping,
+                                combo_set_mapping,
+                                combo,
+                                NULL);
+}
+
 static void
 language_editor_add (PrefsDialog *pd,
                      const char  *code,
@@ -938,7 +1021,7 @@ setup_add_language_dialog (PrefsDialog *dialog)
   guint i;
   GtkBuilder *builder;
 
-  builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/prefs-lang-dialog.ui");
+  builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/gtk/prefs-lang-dialog.ui");
   ad = GTK_WIDGET (gtk_builder_get_object (builder, "add_language_dialog"));
   add_button = GTK_WIDGET (gtk_builder_get_object (builder, "add_button"));
   treeview = GTK_TREE_VIEW (gtk_builder_get_object (builder, "languages_treeview"));
@@ -1264,6 +1347,24 @@ prefs_dialog_response_cb (GtkDialog *widget,
   gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
+static void
+do_not_track_button_clicked_cb (GtkWidget   *button,
+                                PrefsDialog *dialog)
+{
+  char **filters;
+  char **new_filters;
+
+  filters = g_settings_get_strv (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ADBLOCK_FILTERS);
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+    new_filters = ephy_strv_append ((const char * const *)filters, ADBLOCK_PRIVACY_FILTER_URL);
+  else
+    new_filters = ephy_strv_remove ((const char * const *)filters, ADBLOCK_PRIVACY_FILTER_URL);
+  g_settings_set_strv (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_ADBLOCK_FILTERS, (const char * const *)new_filters);
+
+  g_strfreev (filters);
+  g_strfreev (new_filters);
+}
+
 static void
 clear_personal_data_button_clicked_cb (GtkWidget   *button,
                                        PrefsDialog *dialog)
@@ -1621,11 +1722,6 @@ setup_general_page (PrefsDialog *dialog)
                    EPHY_PREFS_WEB_ENABLE_POPUPS,
                    dialog->popups_allow_checkbutton,
                    "active",
-                   G_SETTINGS_BIND_DEFAULT);
-  g_settings_bind (web_settings,
-                   EPHY_PREFS_WEB_ENABLE_ADBLOCK,
-                   dialog->adblock_allow_checkbutton,
-                   "active",
                    G_SETTINGS_BIND_INVERT_BOOLEAN);
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_ENABLE_PLUGINS,
@@ -1633,6 +1729,28 @@ setup_general_page (PrefsDialog *dialog)
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
+  g_settings_bind (web_settings,
+                   EPHY_PREFS_WEB_ENABLE_ADBLOCK,
+                   dialog->adblock_allow_checkbutton,
+                   "active",
+                   G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (web_settings,
+                   EPHY_PREFS_WEB_ENABLE_ADBLOCK,
+                   dialog->do_not_track_checkbutton,
+                   "sensitive",
+                   G_SETTINGS_BIND_DEFAULT);
+
+  g_settings_bind (web_settings,
+                   EPHY_PREFS_WEB_DO_NOT_TRACK,
+                   dialog->do_not_track_checkbutton,
+                   "active",
+                   /* Teensy hack: don't override the previous binding. */
+                   G_SETTINGS_BIND_NO_SENSITIVITY);
+  g_signal_connect (dialog->do_not_track_checkbutton,
+                    "clicked",
+                    G_CALLBACK (do_not_track_button_clicked_cb),
+                    dialog);
+
   create_download_path_button (dialog);
   create_search_engine_combo (GTK_COMBO_BOX (dialog->search_engine_combo));
 }
@@ -1687,7 +1805,7 @@ setup_fonts_page (PrefsDialog *dialog)
 }
 
 static void
-setup_privacy_page (PrefsDialog *dialog)
+setup_stored_data_page (PrefsDialog *dialog)
 {
   GSettings *settings;
   GSettings *web_settings;
@@ -1727,11 +1845,6 @@ setup_privacy_page (PrefsDialog *dialog)
                    dialog->remember_passwords_checkbutton,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
-  g_settings_bind (web_settings,
-                   EPHY_PREFS_WEB_DO_NOT_TRACK,
-                   dialog->do_not_track_checkbutton,
-                   "active",
-                   G_SETTINGS_BIND_DEFAULT);
 
   g_signal_connect (dialog->clear_personal_data_button,
                     "clicked",
@@ -1743,6 +1856,7 @@ static void
 setup_language_page (PrefsDialog *dialog)
 {
   GSettings *web_settings;
+  EphyEncodings *encodings;
 
   web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
 
@@ -1752,6 +1866,10 @@ setup_language_page (PrefsDialog *dialog)
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
+  encodings = EPHY_ENCODINGS (ephy_embed_shell_get_encodings (EPHY_EMBED_SHELL (ephy_shell_get_default ())));
+
+  create_node_combo (dialog, encodings, "ISO-8859-1");
+
   create_language_section (dialog);
 }
 
@@ -1795,7 +1913,7 @@ prefs_dialog_init (PrefsDialog *dialog)
 
   setup_general_page (dialog);
   setup_fonts_page (dialog);
-  setup_privacy_page (dialog);
+  setup_stored_data_page (dialog);
   setup_language_page (dialog);
 #ifdef ENABLE_SYNC
   setup_sync_page (dialog);
