@@ -39,6 +39,7 @@
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
 #include "ephy-sqlite-connection.h"
+#include "ephy-uri-tester-shared.h"
 #include "ephy-web-app-utils.h"
 
 #include <fcntl.h>
@@ -425,6 +426,107 @@ migrate_app_desktop_file_categories (void)
   ephy_web_application_free_application_list (web_apps);
 }
 
+/* https://bugzilla.gnome.org/show_bug.cgi?id=752738 */
+static void
+migrate_insecure_password (SecretItem *item)
+{
+  GHashTable *attributes;
+  WebKitSecurityOrigin *original_origin;
+  const char *original_uri;
+
+  attributes = secret_item_get_attributes (item);
+  original_uri = g_hash_table_lookup (attributes, URI_KEY);
+  original_origin = webkit_security_origin_new_for_uri (original_uri);
+  if (original_origin == NULL) {
+    g_warning ("Failed to convert URI %s to a security origin, insecure password will not be migrated", original_uri);
+    g_hash_table_unref (attributes);
+    return;
+  }
+
+  if (g_strcmp0 (webkit_security_origin_get_protocol (original_origin), "http") == 0) {
+    WebKitSecurityOrigin *new_origin;
+    char *new_uri;
+    GError *error = NULL;
+
+    new_origin = webkit_security_origin_new ("https",
+                                             webkit_security_origin_get_host (original_origin),
+                                             webkit_security_origin_get_port (original_origin));
+    new_uri = webkit_security_origin_to_string (new_origin);
+    webkit_security_origin_unref (new_origin);
+
+    g_hash_table_replace (attributes, g_strdup (URI_KEY), new_uri);
+    secret_item_set_attributes_sync (item, EPHY_FORM_PASSWORD_SCHEMA, attributes, NULL, &error);
+    if (error != NULL) {
+      g_warning ("Failed to convert URI %s to https://, insecure password will not be migrated: %s", original_uri, error->message);
+      g_error_free (error);
+    }
+  }
+
+  g_hash_table_unref (attributes);
+  webkit_security_origin_unref (original_origin);
+}
+
+static void
+migrate_insecure_passwords (void)
+{
+  SecretService *service;
+  GHashTable *attributes;
+  GList *items;
+  int default_profile_migration_version;
+  GError *error = NULL;
+
+  /* This is ephy *profile* migrator. It runs on a per-profile basis. i.e.
+   * each web app runs migrators separately. So this migration step could run
+   * once for a profile dir, then again far in the future when an old web app
+   * is opened. But passwords are global state, not stored in the profile dir,
+   * and we want to run this migration only once. This is tricky to fix, but
+   * it's easier if we relax the constraint to "never run this migrator if it
+   * has been run already for the default profile dir." That's because we don't
+   * really care if a couple web app passwords get converted from insecure to
+   * secure, which is not a big problem and indicates the user probably never
+   * uses Epiphany except for web apps anyway. We just don't want all the user's
+   * passwords to get converted mysteriously because he happens to open a web
+   * app. So check the migration version for the default profile dir and abort
+   * if this migrator has already run there. This way we avoid adding a new flag
+   * file to clutter the profile dir just to check if this migrator has run.
+   */
+  default_profile_migration_version = ephy_profile_utils_get_migration_version_for_profile_dir (ephy_default_dot_dir ());
+  if (default_profile_migration_version >= EPHY_INSECURE_PASSWORDS_MIGRATION_VERSION) {
+    LOG ("Skipping insecure password migration because default profile has already migrated");
+    return;
+  }
+
+  service = secret_service_get_sync (SECRET_SERVICE_LOAD_COLLECTIONS, NULL, &error);
+  if (error != NULL) {
+    g_warning ("Failed to get secret service proxy, insecure passwords will not be migrated: %s", error->message);
+    g_error_free (error);
+    return;
+  }
+
+  attributes = secret_attributes_build (EPHY_FORM_PASSWORD_SCHEMA, NULL);
+
+  items = secret_service_search_sync (service,
+                                      EPHY_FORM_PASSWORD_SCHEMA,
+                                      attributes,
+                                      SECRET_SEARCH_ALL | SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
+                                      NULL,
+                                      &error);
+  if (error != NULL) {
+    g_warning ("Failed to search secret service, insecure passwords will not be migrated: %s", error->message);
+    g_error_free (error);
+    goto out;
+  }
+
+  for (GList *l = items; l != NULL; l = l->next)
+    migrate_insecure_password ((SecretItem *)l->data);
+
+  g_list_free_full (items, g_object_unref);
+
+out:
+  g_object_unref (service);
+  g_hash_table_unref (attributes);
+}
+
 static void
 parse_rdf_lang_tag (xmlNode  *child,
                     xmlChar **value,
@@ -604,6 +706,124 @@ migrate_bookmarks (void)
   g_object_unref (manager);
 }
 
+static void
+migrate_adblock_filters (void)
+{
+  char *adblock_dir;
+  char *filters_filename;
+  char *contents;
+  gsize content_size;
+  GPtrArray *filters_array = NULL;
+  GError *error = NULL;
+
+  adblock_dir = g_build_filename (ephy_dot_dir (), "adblock", NULL);
+  if (!g_file_test (adblock_dir, G_FILE_TEST_IS_DIR)) {
+    g_free (adblock_dir);
+    return;
+  }
+
+  if (!ephy_dot_dir_is_default ()) {
+    char *default_dot_dir;
+
+    /* Adblock filters rules are now shared to save space */
+    ephy_file_delete_dir_recursively (adblock_dir, NULL);
+    g_free (adblock_dir);
+
+    default_dot_dir = ephy_default_dot_dir ();
+    adblock_dir = g_build_filename (default_dot_dir, "adblock", NULL);
+    g_free (default_dot_dir);
+  }
+
+  filters_filename = g_build_filename (adblock_dir, "filters.list", NULL);
+  g_free (adblock_dir);
+
+  if (!g_file_get_contents (filters_filename, &contents, &content_size, &error)) {
+    if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
+      g_warning ("Failed to read filters file: %s: %s", filters_filename, error->message);
+    g_free (filters_filename);
+    g_error_free (error);
+    return;
+  }
+
+  if (content_size > 0) {
+    char **filter_list;
+    guint  filter_list_length;
+    guint  i;
+
+    filter_list = g_strsplit (contents, ";", -1);
+    filter_list_length = g_strv_length (filter_list);
+    if (filter_list_length > 0) {
+      filters_array = g_ptr_array_sized_new (MAX (2, filter_list_length) + 1);
+      g_ptr_array_set_free_func (filters_array, g_free);
+      g_ptr_array_add (filters_array, g_strdup (ADBLOCK_DEFAULT_FILTER_URL));
+
+      for (i = 0; filter_list[i]; i++) {
+        char *url;
+
+        url = g_strstrip (filter_list[i]);
+        if (url[0] != '\0' && !g_str_equal (url, ADBLOCK_DEFAULT_FILTER_URL))
+          g_ptr_array_add (filters_array, g_strdup (url));
+      }
+
+      if (filters_array->len == 1) {
+        /* No additional filters, so do nothing. */
+        g_ptr_array_free (filters_array, TRUE);
+        filters_array = NULL;
+      } else {
+        g_ptr_array_add (filters_array, NULL);
+      }
+    }
+    g_strfreev (filter_list);
+  }
+
+  if (filters_array) {
+    g_settings_set_strv (EPHY_SETTINGS_WEB,
+                         EPHY_PREFS_WEB_ADBLOCK_FILTERS,
+                         (const gchar * const *)filters_array->pdata);
+    g_settings_sync ();
+    g_ptr_array_free (filters_array, TRUE);
+  }
+
+  g_unlink (filters_filename);
+}
+
+static void
+migrate_initial_state (void)
+{
+  char *filename;
+  GFile *file;
+  GError *error = NULL;
+
+  /* EphyInitialState no longer exists. It's just window sizes, so "migrate" it
+   * by simply removing the file to avoid cluttering the profile dir. */
+  filename = g_build_filename (ephy_dot_dir (), "states.xml", NULL);
+  file = g_file_new_for_path (filename);
+
+  g_file_delete (file, NULL, &error);
+  if (error != NULL) {
+    if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
+      g_warning ("Failed to delete %s: %s", filename, error->message);
+    g_error_free (error);
+  }
+
+  g_free (filename);
+  g_object_unref (file);
+}
+
+static void
+migrate_permissions (void)
+{
+  char *filename;
+  GFile *file;
+
+  filename = g_build_filename (ephy_dot_dir (), "hosts.ini", NULL);
+  file = g_file_new_for_path (filename);
+  g_free (filename);
+
+  g_file_delete (file, NULL, NULL);
+  g_object_unref (file);
+}
+
 static void
 migrate_nothing (void)
 {
@@ -615,18 +835,24 @@ migrate_nothing (void)
    */
 }
 
+/* If adding anything here, you need to edit EPHY_PROFILE_MIGRATION_VERSION
+ * in ephy-profile-utils.h. */
 const EphyProfileMigrator migrators[] = {
-  migrate_nothing,
-  migrate_nothing,
-  migrate_nothing,
-  migrate_nothing,
-  migrate_nothing,
-  migrate_nothing,
-  migrate_web_app_links,
-  migrate_new_urls_table,
-  migrate_form_passwords_to_libsecret,
-  migrate_app_desktop_file_categories,
-  migrate_bookmarks,
+  /*  1 */ migrate_nothing,
+  /*  2 */ migrate_nothing,
+  /*  3 */ migrate_nothing,
+  /*  4 */ migrate_nothing,
+  /*  5 */ migrate_nothing,
+  /*  6 */ migrate_nothing,
+  /*  7 */ migrate_web_app_links,
+  /*  8 */ migrate_new_urls_table,
+  /*  9 */ migrate_form_passwords_to_libsecret,
+  /* 10 */ migrate_app_desktop_file_categories,
+  /* 11 */ migrate_insecure_passwords,
+  /* 12 */ migrate_bookmarks,
+  /* 13 */ migrate_adblock_filters,
+  /* 14 */ migrate_initial_state,
+  /* 15 */ migrate_permissions,
 };
 
 static gboolean
@@ -635,6 +861,8 @@ ephy_migrator (void)
   int latest, i;
   EphyProfileMigrator m;
 
+  g_assert (EPHY_PROFILE_MIGRATION_VERSION == G_N_ELEMENTS (migrators));
+
   /* Always try to migrate the data from the old profile dir at the
    * very beginning. */
   migrate_profile_gnome2_to_xdg ();
@@ -649,8 +877,13 @@ ephy_migrator (void)
     if (do_step_n >= EPHY_PROFILE_MIGRATION_VERSION)
       return FALSE;
 
+    if (do_step_n < 1) {
+      g_printf ("Invalid migration step %d\n", do_step_n);
+      return FALSE;
+    }
+
     LOG ("Running only migrator: %d", do_step_n);
-    m = migrators[do_step_n];
+    m = migrators[do_step_n - 1];
     m ();
 
     return TRUE;
