@@ -42,7 +42,7 @@
 #include "ephy-find-toolbar.h"
 #include "ephy-gui.h"
 #include "ephy-header-bar.h"
-#include "ephy-history-window.h"
+#include "ephy-history-dialog.h"
 #include "ephy-link.h"
 #include "ephy-location-entry.h"
 #include "ephy-notebook.h"
@@ -500,14 +500,14 @@ window_cmd_show_history (GSimpleAction *action,
                          GVariant      *parameter,
                          gpointer       user_data)
 {
-  GtkWidget *hwindow;
+  GtkWidget *dialog;
 
-  hwindow = ephy_shell_get_history_window (ephy_shell_get_default ());
+  dialog = ephy_shell_get_history_dialog (ephy_shell_get_default ());
 
-  if (GTK_WINDOW (user_data) != gtk_window_get_transient_for (GTK_WINDOW (hwindow)))
-    gtk_window_set_transient_for (GTK_WINDOW (hwindow),
+  if (GTK_WINDOW (user_data) != gtk_window_get_transient_for (GTK_WINDOW (dialog)))
+    gtk_window_set_transient_for (GTK_WINDOW (dialog),
                                   GTK_WINDOW (user_data));
-  gtk_window_present (GTK_WINDOW (hwindow));
+  gtk_window_present (GTK_WINDOW (dialog));
 }
 
 void
@@ -536,7 +536,7 @@ window_cmd_show_shortcuts (GSimpleAction *action,
   if (shortcuts_window == NULL) {
     GtkBuilder *builder;
 
-    builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/shortcuts-dialog.ui");
+    builder = gtk_builder_new_from_resource ("/org/gnome/epiphany/gtk/shortcuts-dialog.ui");
     shortcuts_window = GTK_WIDGET (gtk_builder_get_object (builder, "shortcuts-dialog"));
 
     g_signal_connect (shortcuts_window,
@@ -693,7 +693,10 @@ window_cmd_reopen_closed_tab (GSimpleAction *action,
                               GVariant      *parameter,
                               gpointer       user_data)
 {
-  ephy_session_undo_close_tab (ephy_shell_get_session (ephy_shell_get_default ()));
+  EphySession *session = ephy_shell_get_session (ephy_shell_get_default ());
+
+  g_assert (session != NULL);
+  ephy_session_undo_close_tab (session);
 }
 
 void
